<?php

namespace Litecms\Logbook\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Logbook\Http\Requests\LogbookRequest;
use Litecms\Logbook\Interfaces\LogbookRepositoryInterface;
use Litecms\Logbook\Models\Logbook;

/**
 * Resource controller class for logbook.
 */
class LogbookResourceController extends BaseController
{

    /**
     * Initialize logbook resource controller.
     *
     * @param type LogbookRepositoryInterface $logbook
     *
     * @return null
     */
    public function __construct(LogbookRepositoryInterface $logbook)
    {
        parent::__construct();
        $this->repository = $logbook;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Logbook\Repositories\Criteria\LogbookResourceCriteria::class);
    }

    /**
     * Display a list of logbook.
     *
     * @return Response
     */
    public function index(LogbookRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Logbook\Repositories\Presenter\LogbookPresenter::class)
                ->$function();
        }

        $logbooks = $this->repository->paginate();

        return $this->response->title(trans('logbook::logbook.names'))
            ->view('logbook::logbook.index', true)
            ->data(compact('logbooks'))
            ->output();
    }

    /**
     * Display logbook.
     *
     * @param Request $request
     * @param Model   $logbook
     *
     * @return Response
     */
    public function show(LogbookRequest $request, Logbook $logbook)
    {

        if ($logbook->exists) {
            $view = 'logbook::logbook.show';
        } else {
            $view = 'logbook::logbook.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('logbook::logbook.name'))
            ->data(compact('logbook'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new logbook.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(LogbookRequest $request)
    {

        $logbook = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('logbook::logbook.name')) 
            ->view('logbook::logbook.create', true) 
            ->data(compact('logbook'))
            ->output();
    }

    /**
     * Create new logbook.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(LogbookRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $logbook                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('logbook::logbook.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('logbook/logbook/' . $logbook->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/logbook/logbook'))
                ->redirect();
        }

    }

    /**
     * Show logbook for editing.
     *
     * @param Request $request
     * @param Model   $logbook
     *
     * @return Response
     */
    public function edit(LogbookRequest $request, Logbook $logbook)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('logbook::logbook.name'))
            ->view('logbook::logbook.edit', true)
            ->data(compact('logbook'))
            ->output();
    }

    /**
     * Update the logbook.
     *
     * @param Request $request
     * @param Model   $logbook
     *
     * @return Response
     */
    public function update(LogbookRequest $request, Logbook $logbook)
    {
        try {
            $attributes = $request->all();

            $logbook->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('logbook::logbook.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('logbook/logbook/' . $logbook->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('logbook/logbook/' . $logbook->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the logbook.
     *
     * @param Model   $logbook
     *
     * @return Response
     */
    public function destroy(LogbookRequest $request, Logbook $logbook)
    {
        try {

            $logbook->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('logbook::logbook.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('logbook/logbook/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('logbook/logbook/' . $logbook->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple logbook.
     *
     * @param Model   $logbook
     *
     * @return Response
     */
    public function delete(LogbookRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('logbook::logbook.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('logbook/logbook'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/logbook/logbook'))
                ->redirect();
        }

    }

    /**
     * Restore deleted logbooks.
     *
     * @param Model   $logbook
     *
     * @return Response
     */
    public function restore(LogbookRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('logbook::logbook.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/logbook/logbook'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/logbook/logbook/'))
                ->redirect();
        }

    }

}
